package org.sonatype.sisu.assembler.gshell.internal;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import javax.inject.Provider;

import org.sonatype.eclipse.bridge.EclipseInstance;

public class EclipseProxy<T>
    implements Provider<T>
{

    private final Class<T> serviceType;

    private final EclipseInstance eclipse;

    EclipseProxy( final Class<T> serviceType, final EclipseInstance eclipse )
    {
        this.serviceType = serviceType;
        this.eclipse = eclipse;
    }

    public T get()
    {
        return (T) Proxy.newProxyInstance( this.getClass().getClassLoader(), new Class[] { serviceType },
            new InvocationHandler()
        {

            public Object invoke( final Object proxy, final Method method, final Object[] args )
                throws Throwable
            {
                try
                {
                    return method.invoke( eclipse.getService( serviceType ), args );
                }
                catch ( final InvocationTargetException e )
                {
                    throw e.getCause();
                }
            }
        } );
    }
}
