/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printHeader;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.codehaus.plexus.util.StringUtils;
import org.sonatype.gshell.command.Command;
import org.sonatype.gshell.command.CommandContext;
import org.sonatype.gshell.command.IO;
import org.sonatype.gshell.file.FileSystemAccess;
import org.sonatype.gshell.util.cli2.Argument;
import org.sonatype.gshell.util.cli2.Option;
import org.sonatype.p2.bridge.IUIdentity;
import org.sonatype.p2.bridge.P2Director;

import com.google.inject.Inject;

@Command( name = "p2/install" )
public class InstallCommand
    extends P2Command
{

    @Argument( required = true )
    private String iu;

    @Option( name = "p", longName = "profile" )
    private final String profile = "master";

    @Option( name = "t", longName = "tag" )
    private String tag;

    @Option( name = "D" )
    private List<String> properties;

    private final FileSystemAccess fileSystem;

    private final P2Director p2Director;

    private final RepositoryRegistry repositoryRegistry;

    private final List<IUIdentity> latestListed;

    @Inject
    public InstallCommand( final FileSystemAccess fileSystem, final P2Director p2Director,
                           final RepositoryRegistry repositoryRegistry,
                           @ListInstallableCommand.Latest final List<IUIdentity> latestListed )
    {
        assert fileSystem != null;
        assert p2Director != null;
        assert repositoryRegistry != null;
        assert latestListed != null;

        this.fileSystem = fileSystem;
        this.p2Director = p2Director;
        this.repositoryRegistry = repositoryRegistry;
        this.latestListed = latestListed;
    }

    public Object execute( final CommandContext context )
        throws Exception
    {
        assert context != null;

        final IO io = context.getIo();

        if ( repositoryRegistry.isEmpty() )
        {
            io.error( "There should be at least one repository configured to be able to install." );
            return Result.FAILURE;
        }

        final File location = getLocation( fileSystem );

        try
        {
            final Integer idx = Integer.valueOf( iu );
            final IUIdentity listed = latestListed.get( idx - 1 );
            iu = listed.identity();
        }
        catch ( final Exception e )
        {
            // do nothing, value is not an index position
        }
        printHeader( io, String.format( "Installing %s:", iu ) );

        final IOLogProxy logProxy = new IOLogProxy( io );

        if ( StringUtils.isBlank( tag ) )
        {
            tag = String.format( "Install %s", iu );
        }
        final List<String> profileProperties = new ArrayList<String>();
        profileProperties.addAll( Arrays.asList( "org.eclipse.equinox.p2.planner.resolveMetaRequirements=false" ) );
        if ( properties != null )
        {
            profileProperties.addAll( properties );
        }
        p2Director.installSingleton( logProxy, location, profile, profileProperties, iu, repositoryRegistry.get(), tag );

        io.println( "" );

        return Result.SUCCESS;
    }
}
