/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.directory;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printHeader;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printNumbered;

import java.io.File;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.List;

import org.sonatype.gshell.command.Command;
import org.sonatype.gshell.command.CommandContext;
import org.sonatype.gshell.command.IO;
import org.sonatype.gshell.file.FileSystemAccess;
import org.sonatype.gshell.util.cli2.Option;
import org.sonatype.p2.bridge.IUIdentity;
import org.sonatype.p2.bridge.P2ProfileRegistry;

import com.google.inject.BindingAnnotation;
import com.google.inject.Inject;

@Command( name = "p2/list/installed" )
public class ListInstalledCommand
    extends P2Command
{

    @Option( name = "p", longName = "profile" )
    private final String profile = "master";

    private final FileSystemAccess fileSystem;

    private final P2ProfileRegistry p2ProfileRegistry;

    private final List<IUIdentity> latest;

    @Inject
    public ListInstalledCommand( final FileSystemAccess fileSystem, final P2ProfileRegistry p2ProfileRegistry,
                                 @Latest final List<IUIdentity> latest )
    {
        assert fileSystem != null;
        assert p2ProfileRegistry != null;
        assert latest != null;
        this.fileSystem = fileSystem;
        this.p2ProfileRegistry = p2ProfileRegistry;
        this.latest = latest;
    }

    public Object execute( final CommandContext context )
        throws Exception
    {
        assert context != null;

        final IO io = context.getIo();

        final File location = getLocation( fileSystem );

        printHeader( io, String.format( "Installed in %s :", directory( location.getAbsolutePath() ) ) );

        final IUIdentity[] installedRoots = p2ProfileRegistry.getInstalledRoots( location, profile );
        latest.clear();

        int i = 1;
        for ( final IUIdentity iu : installedRoots )
        {
            printNumbered( io, i++, installedRoots.length, iu.toString() );
            latest.add( iu );
        }

        io.println( "" );

        return Result.SUCCESS;
    }

    @Retention( RetentionPolicy.RUNTIME )
    @Target( { ElementType.FIELD, ElementType.PARAMETER } )
    @BindingAnnotation
    public @interface Latest
    {
    }

}
