/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import java.io.File;
import java.io.IOException;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.sonatype.gshell.file.FileSystemAccess;

@Singleton
public class LockedFileSystemAccess
    implements FileSystemAccess
{

    private final FileSystemAccess delegate;

    @Inject
    public LockedFileSystemAccess( final FileSystemAccess delegate )
    {
        assert delegate != null;
        this.delegate = delegate;
    }

    public File getShellHomeDir()
        throws IOException
    {
        return delegate.getShellHomeDir();
    }

    public File getUserDir()
        throws IOException
    {
        return getShellHomeDir();
    }

    public File getUserHomeDir()
        throws IOException
    {
        return delegate.getUserHomeDir();
    }

    public boolean hasChildren( final File file )
    {
        return delegate.hasChildren( file );
    }

    public File resolveDir( final String name )
        throws IOException
    {
        return delegate.resolveDir( name );
    }

    public File resolveFile( final String path )
        throws IOException
    {
        return delegate.resolveFile( path );
    }

    public File resolveFile( final File baseDir, final String path )
        throws IOException
    {
        return delegate.resolveFile( baseDir, path );
    }

}
