/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printHeader;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printNumbered;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Collection;
import java.util.List;

import org.sonatype.gshell.command.Command;
import org.sonatype.gshell.command.CommandContext;
import org.sonatype.gshell.command.IO;
import org.sonatype.gshell.command.support.CommandActionSupport;

import com.google.inject.BindingAnnotation;
import com.google.inject.Inject;

@Command( name = "p2/repository/list" )
public class RepositoryListCommand
    extends CommandActionSupport
{

    private final RepositoryRegistry repositoryRegistry;

    private final List<String> latest;

    @Inject
    public RepositoryListCommand( final RepositoryRegistry repositoryRegistry, @Latest final List<String> latest )
    {
        assert repositoryRegistry != null;
        assert latest != null;
        this.repositoryRegistry = repositoryRegistry;
        this.latest = latest;
    }

    public Object execute( final CommandContext context )
        throws Exception
    {
        assert context != null;

        final IO io = context.getIo();

        final Collection<String> repositories = repositoryRegistry.get();

        latest.clear();
        if ( repositories.size() > 0 )
        {
            printHeader( io, "Repositories:" );
            int i = 1;
            for ( final String uri : repositories )
            {
                printNumbered( io, i++, repositories.size(), uri );
                latest.add( uri );
            }
            io.println( "" );
        }
        else
        {
            io.println( "There are no configured repositories" );
        }

        return Result.SUCCESS;
    }

    @Retention( RetentionPolicy.RUNTIME )
    @Target( { ElementType.FIELD, ElementType.PARAMETER } )
    @BindingAnnotation
    public @interface Latest
    {
    }
}
